@@ -1,34 +1,33 @@
 # -*- coding: utf-8 -*-
+"""Periodically store events in a database.
+
+Consuming the events as a stream isn't always suitable
+so this module implements a system to take snapshots of the
+state of a cluster at regular intervals.  There's a full
+implementation of this writing the snapshots to a database
+in :mod:`djcelery.snapshots` in the `django-celery` distribution.
 """
-    celery.events.snapshot
-    ~~~~~~~~~~~~~~~~~~~~~~
-
-    Consuming the events as a stream is not always suitable
-    so this module implements a system to take snapshots of the
-    state of a cluster at regular intervals.  There is a full
-    implementation of this writing the snapshots to a database
-    in :mod:`djcelery.snapshots` in the `django-celery` distribution.
-
-"""
-from __future__ import absolute_import
-
+from __future__ import absolute_import, print_function, unicode_literals
 from kombu.utils.limits import TokenBucket
-
 from celery import platforms
 from celery.app import app_or_default
-from celery.utils import timer2
+from celery.utils.timer2 import Timer
 from celery.utils.dispatch import Signal
 from celery.utils.imports import instantiate
 from celery.utils.log import get_logger
-from celery.utils.timeutils import rate
+from celery.utils.time import rate
+
+__all__ = ['Polaroid', 'evcam']
 
 logger = get_logger('celery.evcam')
 
 
 class Polaroid(object):
-    timer = timer2
-    shutter_signal = Signal(providing_args=('state', ))
-    cleanup_signal = Signal()
+    """Record event snapshots."""
+
+    timer = None
+    shutter_signal = Signal(name='shutter_signal', providing_args={'state'})
+    cleanup_signal = Signal(name='cleanup_signal')
     clear_after = False
 
     _tref = None
@@ -40,15 +39,15 @@ class Polaroid(object):
         self.state = state
         self.freq = freq
         self.cleanup_freq = cleanup_freq
-        self.timer = timer or self.timer
+        self.timer = timer or self.timer or Timer()
         self.logger = logger
         self.maxrate = maxrate and TokenBucket(rate(maxrate))
 
     def install(self):
-        self._tref = self.timer.apply_interval(self.freq * 1000.0,
-                                               self.capture)
-        self._ctref = self.timer.apply_interval(self.cleanup_freq * 1000.0,
-                                                self.cleanup)
+        self._tref = self.timer.call_repeatedly(self.freq, self.capture)
+        self._ctref = self.timer.call_repeatedly(
+            self.cleanup_freq, self.cleanup,
+        )
 
     def on_shutter(self, state):
         pass
@@ -58,13 +57,13 @@ class Polaroid(object):
 
     def cleanup(self):
         logger.debug('Cleanup: Running...')
-        self.cleanup_signal.send(None)
+        self.cleanup_signal.send(sender=self.state)
         self.on_cleanup()
 
     def shutter(self):
         if self.maxrate is None or self.maxrate.can_consume():
             logger.debug('Shutter: %s', self.state)
-            self.shutter_signal.send(self.state)
+            self.shutter_signal.send(sender=self.state)
             self.on_shutter(self.state)
 
     def capture(self):
@@ -87,6 +86,7 @@ class Polaroid(object):
 
 def evcam(camera, freq=1.0, maxrate=None, loglevel=0,
           logfile=None, pidfile=None, timer=None, app=None):
+    """Start snapshot recorder."""
     app = app_or_default(app)
 
     if pidfile:
@@ -94,14 +94,13 @@ def evcam(camera, freq=1.0, maxrate=None, loglevel=0,
 
     app.log.setup_logging_subsystem(loglevel, logfile)
 
-    logger.info(
-        '-> evcam: Taking snapshots with %s (every %s secs.)\n' % (
-            camera, freq))
+    print('-> evcam: Taking snapshots with {0} (every {1} secs.)'.format(
+        camera, freq))
     state = app.events.State()
     cam = instantiate(camera, state, app=app, freq=freq,
                       maxrate=maxrate, timer=timer)
     cam.install()
-    conn = app.connection()
+    conn = app.connection_for_read()
     recv = app.events.Receiver(conn, handlers={'*': state.event})
     try:
         try:
