@@ -1,27 +1,24 @@
 # -*- coding: utf-8 -*-
-"""
-    celery.worker.components
-    ~~~~~~~~~~~~~~~~~~~~~~~~
-
-    Default worker bootsteps.
-
-"""
-from __future__ import absolute_import
+"""Worker-level Bootsteps."""
+from __future__ import absolute_import, unicode_literals
 
 import atexit
 import warnings
 
-from kombu.asynchronous import Hub as _Hub, get_event_loop, set_event_loop
-from kombu.asynchronous.semaphore import DummyLock, LaxBoundedSemaphore
-from kombu.asynchronous.timer import Timer as _Timer
+from kombu.async import Hub as _Hub, get_event_loop, set_event_loop
+from kombu.async.semaphore import DummyLock, LaxBoundedSemaphore
+from kombu.async.timer import Timer as _Timer
 
 from celery import bootsteps
 from celery._state import _set_task_join_will_block
 from celery.exceptions import ImproperlyConfigured
 from celery.five import string_t
+from celery.platforms import IS_WINDOWS
 from celery.utils.log import worker_logger as logger
 
-__all__ = ['Timer', 'Hub', 'Queues', 'Pool', 'Beat', 'StateDB', 'Consumer']
+__all__ = ['Timer', 'Hub', 'Pool', 'Beat', 'StateDB', 'Consumer']
+
+GREEN_POOLS = {'eventlet', 'gevent'}
 
 ERR_B_GREEN = """\
 -B option doesn't work with eventlet/gevent pools: \
@@ -29,14 +26,14 @@ use standalone beat instead.\
 """
 
 W_POOL_SETTING = """
-The CELERYD_POOL setting should not be used to select the eventlet/gevent
+The worker_pool setting shouldn't be used to select the eventlet/gevent
 pools, instead you *must use the -P* argument so that patches are applied
 as early as possible.
 """
 
 
 class Timer(bootsteps.Step):
-    """This step initializes the internal timer used by the worker."""
+    """Timer bootstep."""
 
     def create(self, w):
         if w.use_eventloop:
@@ -44,26 +41,29 @@ class Timer(bootsteps.Step):
             w.timer = _Timer(max_interval=10.0)
         else:
             if not w.timer_cls:
-                # Default Timer is set by the pool, as e.g. eventlet
-                # needs a custom implementation.
+                # Default Timer is set by the pool, as for example, the
+                # eventlet pool needs a custom timer implementation.
                 w.timer_cls = w.pool_cls.Timer
             w.timer = self.instantiate(w.timer_cls,
                                        max_interval=w.timer_precision,
-                                       on_timer_error=self.on_timer_error,
-                                       on_timer_tick=self.on_timer_tick)
+                                       on_error=self.on_timer_error,
+                                       on_tick=self.on_timer_tick)
 
     def on_timer_error(self, exc):
         logger.error('Timer error: %r', exc, exc_info=True)
 
     def on_timer_tick(self, delay):
-        logger.debug('Timer wake-up! Next eta %s secs.', delay)
+        logger.debug('Timer wake-up! Next ETA %s secs.', delay)
 
 
 class Hub(bootsteps.StartStopStep):
-    requires = (Timer, )
+    """Worker starts the event loop."""
+
+    requires = (Timer,)
 
     def __init__(self, w, **kwargs):
         w.hub = None
+        super(Hub, self).__init__(w, **kwargs)
 
     def include_if(self, w):
         return w.use_eventloop
@@ -71,7 +71,9 @@ class Hub(bootsteps.StartStopStep):
     def create(self, w):
         w.hub = get_event_loop()
         if w.hub is None:
-            w.hub = set_event_loop(_Hub(w.timer))
+            required_hub = getattr(w._conninfo, 'requires_hub', None)
+            w.hub = set_event_loop((
+                required_hub if required_hub else _Hub)(w.timer))
         self._patch_thread_primitives(w)
         return self
 
@@ -90,30 +92,17 @@ class Hub(bootsteps.StartStopStep):
         # multiprocessing's ApplyResult uses this lock.
         try:
             from billiard import pool
-        except ImportError:
+        except ImportError:  # pragma: no cover
             pass
         else:
             pool.Lock = DummyLock
 
 
-class Queues(bootsteps.Step):
-    """This bootstep initializes the internal queues
-    used by the worker."""
-    label = 'Queues (intra)'
-    requires = (Hub, )
-
-    def create(self, w):
-        w.process_task = w._process_task
-        if w.use_eventloop:
-            if w.pool_putlocks and w.pool_cls.uses_semaphore:
-                w.process_task = w._process_task_sem
-
-
 class Pool(bootsteps.StartStopStep):
     """Bootstep managing the worker pool.
 
     Describes how to initialize the worker pool, and starts and stops
-    the pool during worker startup/shutdown.
+    the pool during worker start-up/shutdown.
 
     Adds attributes:
 
@@ -121,24 +110,22 @@ class Pool(bootsteps.StartStopStep):
         * pool
         * max_concurrency
         * min_concurrency
-
     """
-    requires = (Queues, )
 
-    def __init__(self, w, autoscale=None, autoreload=None,
-                 no_execv=False, optimization=None, **kwargs):
+    requires = (Hub,)
+
+    def __init__(self, w, autoscale=None, **kwargs):
+        w.pool = None
+        w.max_concurrency = None
+        w.min_concurrency = w.concurrency
+        self.optimization = w.optimization
         if isinstance(autoscale, string_t):
             max_c, _, min_c = autoscale.partition(',')
             autoscale = [int(max_c), min_c and int(min_c) or 0]
         w.autoscale = autoscale
-        w.pool = None
-        w.max_concurrency = None
-        w.min_concurrency = w.concurrency
-        w.no_execv = no_execv
         if w.autoscale:
             w.max_concurrency, w.min_concurrency = w.autoscale
-        self.autoreload_enabled = autoreload
-        self.optimization = optimization
+        super(Pool, self).__init__(w, **kwargs)
 
     def close(self, w):
         if w.pool:
@@ -148,32 +135,38 @@ class Pool(bootsteps.StartStopStep):
         if w.pool:
             w.pool.terminate()
 
-    def create(self, w, semaphore=None, max_restarts=None):
-        if w.app.conf.CELERYD_POOL in ('eventlet', 'gevent'):
+    def create(self, w):
+        semaphore = None
+        max_restarts = None
+        if w.app.conf.worker_pool in GREEN_POOLS:  # pragma: no cover
             warnings.warn(UserWarning(W_POOL_SETTING))
-        threaded = not w.use_eventloop
+        threaded = not w.use_eventloop or IS_WINDOWS
         procs = w.min_concurrency
-        forking_enable = w.no_execv if w.force_execv else True
+        w.process_task = w._process_task
         if not threaded:
             semaphore = w.semaphore = LaxBoundedSemaphore(procs)
             w._quick_acquire = w.semaphore.acquire
             w._quick_release = w.semaphore.release
             max_restarts = 100
-        allow_restart = self.autoreload_enabled or w.pool_restarts
+            if w.pool_putlocks and w.pool_cls.uses_semaphore:
+                w.process_task = w._process_task_sem
+        allow_restart = w.pool_restarts
         pool = w.pool = self.instantiate(
             w.pool_cls, w.min_concurrency,
             initargs=(w.app, w.hostname),
             maxtasksperchild=w.max_tasks_per_child,
-            timeout=w.task_time_limit,
-            soft_timeout=w.task_soft_time_limit,
+            max_memory_per_child=w.max_memory_per_child,
+            timeout=w.time_limit,
+            soft_timeout=w.soft_time_limit,
             putlocks=w.pool_putlocks and threaded,
             lost_worker_timeout=w.worker_lost_wait,
             threads=threaded,
             max_restarts=max_restarts,
             allow_restart=allow_restart,
-            forking_enable=forking_enable,
+            forking_enable=True,
             semaphore=semaphore,
             sched_strategy=self.optimization,
+            app=w.app,
         )
         _set_task_join_will_block(pool.task_join_will_block)
         return pool
@@ -188,16 +181,16 @@ class Pool(bootsteps.StartStopStep):
 class Beat(bootsteps.StartStopStep):
     """Step used to embed a beat process.
 
-    This will only be enabled if the ``beat``
-    argument is set.
-
+    Enabled when the ``beat`` argument is set.
     """
+
     label = 'Beat'
     conditional = True
 
     def __init__(self, w, beat=False, **kwargs):
         self.enabled = w.beat = beat
         w.beat = None
+        super(Beat, self).__init__(w, beat=beat, **kwargs)
 
     def create(self, w):
         from celery.beat import EmbeddedService
@@ -205,23 +198,26 @@ class Beat(bootsteps.StartStopStep):
             raise ImproperlyConfigured(ERR_B_GREEN)
         b = w.beat = EmbeddedService(w.app,
                                      schedule_filename=w.schedule_filename,
-                                     scheduler_cls=w.scheduler_cls)
+                                     scheduler_cls=w.scheduler)
         return b
 
 
 class StateDB(bootsteps.Step):
-    """This bootstep sets up the workers state db if enabled."""
+    """Bootstep that sets up between-restart state database file."""
 
     def __init__(self, w, **kwargs):
-        self.enabled = w.state_db
+        self.enabled = w.statedb
         w._persistence = None
+        super(StateDB, self).__init__(w, **kwargs)
 
     def create(self, w):
-        w._persistence = w.state.Persistent(w.state, w.state_db, w.app.clock)
+        w._persistence = w.state.Persistent(w.state, w.statedb, w.app.clock)
         atexit.register(w._persistence.save)
 
 
 class Consumer(bootsteps.StartStopStep):
+    """Bootstep starting the Consumer blueprint."""
+
     last = True
 
     def create(self, w):
@@ -232,7 +228,7 @@ class Consumer(bootsteps.StartStopStep):
         c = w.consumer = self.instantiate(
             w.consumer_cls, w.process_task,
             hostname=w.hostname,
-            send_events=w.send_events,
+            task_events=w.task_events,
             init_callback=w.ready_callback,
             initial_prefetch_count=prefetch_count,
             pool=w.pool,
