@@ -1,6 +1,6 @@
 """myapp.py
 
-Usage:
+Usage::
 
    (window1)$ python myapp.py worker -l info
 
@@ -22,17 +22,18 @@ name using the fully qualified form::
     $ celery -A myapp:app worker -l info
 
 """
+from __future__ import absolute_import, unicode_literals
 from celery import Celery
 
 app = Celery(
     'myapp',
     broker='amqp://guest@localhost//',
-    # ### add result backend here if needed.
+    # ## add result backend here if needed.
     # backend='rpc'
 )
 
 
-@app.task()
+@app.task
 def add(x, y):
     return x + y
 
