@@ -1,15 +1,14 @@
-from __future__ import absolute_import
-
+from __future__ import absolute_import, unicode_literals
 from celery import Celery
 
-app = Celery('proj.celery',
+app = Celery('proj',
              broker='amqp://',
              backend='amqp://',
              include=['proj.tasks'])
 
 # Optional configuration, see the application user guide.
 app.conf.update(
-    CELERY_TASK_RESULT_EXPIRES=3600,
+    result_expires=3600,
 )
 
 if __name__ == '__main__':
