@@ -13,12 +13,11 @@
 #
 # Joining the graph asynchronously with a callback
 # (Note: only two levels, the deps are considered final
-#        when the second task is ready.)
+#        when the second task is ready).
 #
 #    >>> unlock_graph.apply_async((A.apply_async(),
-#    ...                           A_callback.subtask()), countdown=1)
-
-
+#    ...                           A_callback.s()), countdown=1)
+from __future__ import absolute_import, print_function, unicode_literals
 from celery import chord, group, task, signature, uuid
 from celery.result import AsyncResult, ResultSet, allow_join_result
 from collections import deque
@@ -31,13 +30,13 @@ def add(x, y):
 
 @task()
 def make_request(id, url):
-    print('GET {0!r}'.format(url))
+    print('-get: {0!r}'.format(url))
     return url
 
 
 @task()
 def B_callback(urls, id):
-    print('batch {0} done'.format(id))
+    print('-batch {0} done'.format(id))
     return urls
 
 
@@ -88,7 +87,7 @@ def unlock_graph(result, callback,
 
 @task()
 def A_callback(res):
-    print('Everything is done: {0!r}'.format(res))
+    print('-everything done: {0!r}'.format(res))
     return res
 
 
