@@ -1,5 +1,4 @@
-from __future__ import absolute_import
-
+from __future__ import absolute_import, unicode_literals
 from celery import Celery
 
 app = Celery('tasks', broker='amqp://')
