@@ -165,7 +165,7 @@ class test_head_from_fun:
     @skip.unless_python3()
     def test_regression_3678(self):
         local = {}
-        fun = ('def f(foo, *args, bar=""):'
+        fun = ('def f(foo, *args, bar="", **kwargs):'
                '    return foo, args, bar')
         exec(fun, {}, local)
 
@@ -205,6 +205,18 @@ class test_head_from_fun:
         g(a=1, b=2)
         g(a=1, b=2, c=3)
 
+    def test_classmethod(self):
+        class A(object):
+            @classmethod
+            def f(cls, x):
+                return x
+
+        fun = head_from_fun(A.f, bound=False)
+        assert fun(A, 1) == 1
+
+        fun = head_from_fun(A.f, bound=True)
+        assert fun(1) == 1
+
 
 class test_fun_takes_argument:
 
